/*
 * This file was generated by orbit-idl-2 - DO NOT EDIT!
 */

#include <string.h>
#define ORBIT2_STUBS_API
#include "Evolution-DataServer-Mail.h"

Evolution_Mail_Buffer* Evolution_Mail_MessageStream_next(Evolution_Mail_MessageStream _obj, const CORBA_long size, CORBA_Environment *ev){
Evolution_Mail_Buffer* _ORBIT_retval;
gpointer _args[1];
_args[0] = (gpointer)&size;
ORBit_c_stub_invoke (_obj, &Evolution_Mail_MessageStream__iinterface.methods, 0, &_ORBIT_retval, _args, NULL, ev, Evolution_Mail_MessageStream__classid, G_STRUCT_OFFSET (POA_Evolution_Mail_MessageStream__epv, next),
(ORBitSmallSkeleton) _ORBIT_skel_small_Evolution_Mail_MessageStream_next);

return _ORBIT_retval;
}
void Evolution_Mail_MessageStream_dispose(Evolution_Mail_MessageStream _obj, CORBA_Environment *ev){
ORBit_c_stub_invoke (_obj, &Evolution_Mail_MessageStream__iinterface.methods, 1, NULL, NULL, NULL, ev, Evolution_Mail_MessageStream__classid, G_STRUCT_OFFSET (POA_Evolution_Mail_MessageStream__epv, dispose),
(ORBitSmallSkeleton) _ORBIT_skel_small_Evolution_Mail_MessageStream_dispose);

}
Evolution_Mail_MessageInfos* Evolution_Mail_MessageIterator_next(Evolution_Mail_MessageIterator _obj, const CORBA_long limit, CORBA_Environment *ev){
Evolution_Mail_MessageInfos* _ORBIT_retval;
gpointer _args[1];
_args[0] = (gpointer)&limit;
ORBit_c_stub_invoke (_obj, &Evolution_Mail_MessageIterator__iinterface.methods, 0, &_ORBIT_retval, _args, NULL, ev, Evolution_Mail_MessageIterator__classid, G_STRUCT_OFFSET (POA_Evolution_Mail_MessageIterator__epv, next),
(ORBitSmallSkeleton) _ORBIT_skel_small_Evolution_Mail_MessageIterator_next);

return _ORBIT_retval;
}
void Evolution_Mail_MessageIterator_dispose(Evolution_Mail_MessageIterator _obj, CORBA_Environment *ev){
ORBit_c_stub_invoke (_obj, &Evolution_Mail_MessageIterator__iinterface.methods, 1, NULL, NULL, NULL, ev, Evolution_Mail_MessageIterator__classid, G_STRUCT_OFFSET (POA_Evolution_Mail_MessageIterator__epv, dispose),
(ORBitSmallSkeleton) _ORBIT_skel_small_Evolution_Mail_MessageIterator_dispose);

}
void Evolution_Mail_SessionListener_changed(Evolution_Mail_SessionListener _obj, const Evolution_Mail_Session session, const Evolution_Mail_SessionChanges* changes, CORBA_Environment *ev){
gpointer _args[2];
_args[0] = (gpointer)&session;
_args[1] = (gpointer)changes;
ORBit_c_stub_invoke (_obj, &Evolution_Mail_SessionListener__iinterface.methods, 0, NULL, _args, NULL, ev, Evolution_Mail_SessionListener__classid, G_STRUCT_OFFSET (POA_Evolution_Mail_SessionListener__epv, changed),
(ORBitSmallSkeleton) _ORBIT_skel_small_Evolution_Mail_SessionListener_changed);

}
void Evolution_Mail_SessionListener_shutdown(Evolution_Mail_SessionListener _obj, const Evolution_Mail_Session session, CORBA_Environment *ev){
gpointer _args[1];
_args[0] = (gpointer)&session;
ORBit_c_stub_invoke (_obj, &Evolution_Mail_SessionListener__iinterface.methods, 1, NULL, _args, NULL, ev, Evolution_Mail_SessionListener__classid, G_STRUCT_OFFSET (POA_Evolution_Mail_SessionListener__epv, shutdown),
(ORBitSmallSkeleton) _ORBIT_skel_small_Evolution_Mail_SessionListener_shutdown);

}
void Evolution_Mail_StoreListener_changed(Evolution_Mail_StoreListener _obj, const Evolution_Mail_Store store, const Evolution_Mail_StoreChanges* changes, CORBA_Environment *ev){
gpointer _args[2];
_args[0] = (gpointer)&store;
_args[1] = (gpointer)changes;
ORBit_c_stub_invoke (_obj, &Evolution_Mail_StoreListener__iinterface.methods, 0, NULL, _args, NULL, ev, Evolution_Mail_StoreListener__classid, G_STRUCT_OFFSET (POA_Evolution_Mail_StoreListener__epv, changed),
(ORBitSmallSkeleton) _ORBIT_skel_small_Evolution_Mail_StoreListener_changed);

}
void Evolution_Mail_FolderListener_changed(Evolution_Mail_FolderListener _obj, const Evolution_Mail_Folder folder, const Evolution_Mail_FolderChanges* changes, CORBA_Environment *ev){
gpointer _args[2];
_args[0] = (gpointer)&folder;
_args[1] = (gpointer)changes;
ORBit_c_stub_invoke (_obj, &Evolution_Mail_FolderListener__iinterface.methods, 0, NULL, _args, NULL, ev, Evolution_Mail_FolderListener__classid, G_STRUCT_OFFSET (POA_Evolution_Mail_FolderListener__epv, changed),
(ORBitSmallSkeleton) _ORBIT_skel_small_Evolution_Mail_FolderListener_changed);

}
CORBA_boolean Evolution_Mail_Session_getProperties(Evolution_Mail_Session _obj, const Evolution_Mail_PropertyNames* names, Evolution_Mail_Properties** props, CORBA_Environment *ev){
CORBA_boolean _ORBIT_retval;
gpointer _args[2];
_args[0] = (gpointer)names;
_args[1] = props;
ORBit_c_stub_invoke (_obj, &Evolution_Mail_Session__iinterface.methods, 0, &_ORBIT_retval, _args, NULL, ev, Evolution_Mail_Session__classid, G_STRUCT_OFFSET (POA_Evolution_Mail_Session__epv, getProperties),
(ORBitSmallSkeleton) _ORBIT_skel_small_Evolution_Mail_Session_getProperties);

return _ORBIT_retval;
}
Evolution_Mail_StoreInfos* Evolution_Mail_Session_getStores(Evolution_Mail_Session _obj, const CORBA_char * pattern, const Evolution_Mail_StoreListener listener, CORBA_Environment *ev){
Evolution_Mail_StoreInfos* _ORBIT_retval;
gpointer _args[2];
_args[0] = (gpointer)&pattern;
_args[1] = (gpointer)&listener;
ORBit_c_stub_invoke (_obj, &Evolution_Mail_Session__iinterface.methods, 1, &_ORBIT_retval, _args, NULL, ev, Evolution_Mail_Session__classid, G_STRUCT_OFFSET (POA_Evolution_Mail_Session__epv, getStores),
(ORBitSmallSkeleton) _ORBIT_skel_small_Evolution_Mail_Session_getStores);

return _ORBIT_retval;
}
void Evolution_Mail_Session_addListener(Evolution_Mail_Session _obj, const Evolution_Mail_SessionListener listener, CORBA_Environment *ev){
gpointer _args[1];
_args[0] = (gpointer)&listener;
ORBit_c_stub_invoke (_obj, &Evolution_Mail_Session__iinterface.methods, 2, NULL, _args, NULL, ev, Evolution_Mail_Session__classid, G_STRUCT_OFFSET (POA_Evolution_Mail_Session__epv, addListener),
(ORBitSmallSkeleton) _ORBIT_skel_small_Evolution_Mail_Session_addListener);

}
void Evolution_Mail_Session_removeListener(Evolution_Mail_Session _obj, const Evolution_Mail_SessionListener listener, CORBA_Environment *ev){
gpointer _args[1];
_args[0] = (gpointer)&listener;
ORBit_c_stub_invoke (_obj, &Evolution_Mail_Session__iinterface.methods, 3, NULL, _args, NULL, ev, Evolution_Mail_Session__classid, G_STRUCT_OFFSET (POA_Evolution_Mail_Session__epv, removeListener),
(ORBitSmallSkeleton) _ORBIT_skel_small_Evolution_Mail_Session_removeListener);

}
CORBA_boolean Evolution_Mail_Store_getProperties(Evolution_Mail_Store _obj, const Evolution_Mail_PropertyNames* names, Evolution_Mail_Properties** props, CORBA_Environment *ev){
CORBA_boolean _ORBIT_retval;
gpointer _args[2];
_args[0] = (gpointer)names;
_args[1] = props;
ORBit_c_stub_invoke (_obj, &Evolution_Mail_Store__iinterface.methods, 0, &_ORBIT_retval, _args, NULL, ev, Evolution_Mail_Store__classid, G_STRUCT_OFFSET (POA_Evolution_Mail_Store__epv, getProperties),
(ORBitSmallSkeleton) _ORBIT_skel_small_Evolution_Mail_Store_getProperties);

return _ORBIT_retval;
}
Evolution_Mail_FolderInfos* Evolution_Mail_Store_getFolders(Evolution_Mail_Store _obj, const CORBA_char * pattern, const Evolution_Mail_FolderListener listener, CORBA_Environment *ev){
Evolution_Mail_FolderInfos* _ORBIT_retval;
gpointer _args[2];
_args[0] = (gpointer)&pattern;
_args[1] = (gpointer)&listener;
ORBit_c_stub_invoke (_obj, &Evolution_Mail_Store__iinterface.methods, 1, &_ORBIT_retval, _args, NULL, ev, Evolution_Mail_Store__classid, G_STRUCT_OFFSET (POA_Evolution_Mail_Store__epv, getFolders),
(ORBitSmallSkeleton) _ORBIT_skel_small_Evolution_Mail_Store_getFolders);

return _ORBIT_retval;
}
void Evolution_Mail_Store_sendMessage(Evolution_Mail_Store _obj, const Evolution_Mail_MessageStream msg, CORBA_Environment *ev){
gpointer _args[1];
_args[0] = (gpointer)&msg;
ORBit_c_stub_invoke (_obj, &Evolution_Mail_Store__iinterface.methods, 2, NULL, _args, NULL, ev, Evolution_Mail_Store__classid, G_STRUCT_OFFSET (POA_Evolution_Mail_Store__epv, sendMessage),
(ORBitSmallSkeleton) _ORBIT_skel_small_Evolution_Mail_Store_sendMessage);

}
CORBA_boolean Evolution_Mail_Folder_getProperties(Evolution_Mail_Folder _obj, const Evolution_Mail_PropertyNames* names, Evolution_Mail_Properties** props, CORBA_Environment *ev){
CORBA_boolean _ORBIT_retval;
gpointer _args[2];
_args[0] = (gpointer)names;
_args[1] = props;
ORBit_c_stub_invoke (_obj, &Evolution_Mail_Folder__iinterface.methods, 0, &_ORBIT_retval, _args, NULL, ev, Evolution_Mail_Folder__classid, G_STRUCT_OFFSET (POA_Evolution_Mail_Folder__epv, getProperties),
(ORBitSmallSkeleton) _ORBIT_skel_small_Evolution_Mail_Folder_getProperties);

return _ORBIT_retval;
}
Evolution_Mail_MessageIterator Evolution_Mail_Folder_getMessages(Evolution_Mail_Folder _obj, const CORBA_char * pattern, CORBA_Environment *ev){
Evolution_Mail_MessageIterator _ORBIT_retval;
gpointer _args[1];
_args[0] = (gpointer)&pattern;
ORBit_c_stub_invoke (_obj, &Evolution_Mail_Folder__iinterface.methods, 1, &_ORBIT_retval, _args, NULL, ev, Evolution_Mail_Folder__classid, G_STRUCT_OFFSET (POA_Evolution_Mail_Folder__epv, getMessages),
(ORBitSmallSkeleton) _ORBIT_skel_small_Evolution_Mail_Folder_getMessages);

return _ORBIT_retval;
}
void Evolution_Mail_Folder_changeMessages(Evolution_Mail_Folder _obj, const Evolution_Mail_MessageInfoSets* infos, CORBA_Environment *ev){
gpointer _args[1];
_args[0] = (gpointer)infos;
ORBit_c_stub_invoke (_obj, &Evolution_Mail_Folder__iinterface.methods, 2, NULL, _args, NULL, ev, Evolution_Mail_Folder__classid, G_STRUCT_OFFSET (POA_Evolution_Mail_Folder__epv, changeMessages),
(ORBitSmallSkeleton) _ORBIT_skel_small_Evolution_Mail_Folder_changeMessages);

}
Evolution_Mail_MessageStream Evolution_Mail_Folder_getMessage(Evolution_Mail_Folder _obj, const CORBA_char * uid, CORBA_Environment *ev){
Evolution_Mail_MessageStream _ORBIT_retval;
gpointer _args[1];
_args[0] = (gpointer)&uid;
ORBit_c_stub_invoke (_obj, &Evolution_Mail_Folder__iinterface.methods, 3, &_ORBIT_retval, _args, NULL, ev, Evolution_Mail_Folder__classid, G_STRUCT_OFFSET (POA_Evolution_Mail_Folder__epv, getMessage),
(ORBitSmallSkeleton) _ORBIT_skel_small_Evolution_Mail_Folder_getMessage);

return _ORBIT_retval;
}
void Evolution_Mail_Folder_appendMessage(Evolution_Mail_Folder _obj, const Evolution_Mail_MessageInfoSet* info, const Evolution_Mail_MessageStream msg, CORBA_Environment *ev){
gpointer _args[2];
_args[0] = (gpointer)info;
_args[1] = (gpointer)&msg;
ORBit_c_stub_invoke (_obj, &Evolution_Mail_Folder__iinterface.methods, 4, NULL, _args, NULL, ev, Evolution_Mail_Folder__classid, G_STRUCT_OFFSET (POA_Evolution_Mail_Folder__epv, appendMessage),
(ORBitSmallSkeleton) _ORBIT_skel_small_Evolution_Mail_Folder_appendMessage);

}
