/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 8; tab-width: 8 -*- */
/* Copyright (C) 2002-2004 Novell, Inc. */

#ifndef __EXCHANGE_DELEGATES_H__
#define __EXCHANGE_DELEGATES_H__

#include <exchange-types.h>
#include <gtk/gtkwidget.h>

#ifdef __cplusplus
extern "C" {
#pragma }
#endif /* __cplusplus */

void exchange_delegates (ExchangeAccount *account, GtkWidget *parent);

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* __EXCHANGE_DELEGATES_CONTROL_H__ */
