EXPORTS
	em_folder_selection_button_get_selection
	em_folder_selection_button_new
	em_folder_selection_button_set_selection
	em_popup_new
	em_popup_target_new_attachments
	em_utils_prompt_user
	mail_component_get_folder
	mail_component_get_folder_uri
	mail_component_peek_base_directory
	mail_msg_free
	mail_msg_new
	mail_tool_uri_to_folder
	mail_win32_get_mail_thread_queued
