
#ifndef __evolution_mail_marshal_MARSHAL_H__
#define __evolution_mail_marshal_MARSHAL_H__

#include	<glib-object.h>

G_BEGIN_DECLS

/* VOID:POINTER,POINTER (evolution-mail-marshal.list:1) */
extern void evolution_mail_marshal_VOID__POINTER_POINTER (GClosure     *closure,
                                                          GValue       *return_value,
                                                          guint         n_param_values,
                                                          const GValue *param_values,
                                                          gpointer      invocation_hint,
                                                          gpointer      marshal_data);

/* VOID:POINTER (evolution-mail-marshal.list:2) */
#define evolution_mail_marshal_VOID__POINTER	g_cclosure_marshal_VOID__POINTER

G_END_DECLS

#endif /* __evolution_mail_marshal_MARSHAL_H__ */

