
struct _EImportImporter *evolution_ldif_importer_peek(void);
struct _EImportImporter *evolution_vcard_importer_peek(void);
struct _EImportImporter *evolution_csv_outlook_importer_peek(void);
struct _EImportImporter *evolution_csv_mozilla_importer_peek(void);
struct _EImportImporter *evolution_csv_evolution_importer_peek(void);
