/*
 * This file was generated by orbit-idl-2 - DO NOT EDIT!
 */

#include <string.h>
#define ORBIT2_STUBS_API
#include "Evolution-DataServer-Mail.h"

static ORBitSmallSkeleton get_skel_small_Evolution_Mail_MessageStream(POA_Evolution_Mail_MessageStream *servant,
const char *opname,gpointer *m_data, gpointer *impl)
{
switch(opname[0]) {
case 'd':
if(strcmp((opname + 1), "ispose")) break;
*impl = (gpointer)servant->vepv->Evolution_Mail_MessageStream_epv->dispose;
{ORBit_IInterface *volatile _t_=&Evolution_Mail_MessageStream__iinterface;*m_data = (gpointer)&_t_->methods._buffer [1];}
return (ORBitSmallSkeleton)_ORBIT_skel_small_Evolution_Mail_MessageStream_dispose;
break;
case 'n':
if(strcmp((opname + 1), "ext")) break;
*impl = (gpointer)servant->vepv->Evolution_Mail_MessageStream_epv->next;
{ORBit_IInterface *volatile _t_=&Evolution_Mail_MessageStream__iinterface;*m_data = (gpointer)&_t_->methods._buffer [0];}
return (ORBitSmallSkeleton)_ORBIT_skel_small_Evolution_Mail_MessageStream_next;
break;
case 'q':
if(strcmp((opname + 1), "ueryInterface")) break;
*impl = (gpointer)servant->vepv->Bonobo_Unknown_epv->queryInterface;
{ORBit_IInterface *volatile _t_=&Bonobo_Unknown__iinterface;*m_data = (gpointer)&_t_->methods._buffer [2];}
return (ORBitSmallSkeleton)_ORBIT_skel_small_Bonobo_Unknown_queryInterface;
break;
case 'r':
if(strcmp((opname + 1), "ef")) break;
*impl = (gpointer)servant->vepv->Bonobo_Unknown_epv->ref;
{ORBit_IInterface *volatile _t_=&Bonobo_Unknown__iinterface;*m_data = (gpointer)&_t_->methods._buffer [0];}
return (ORBitSmallSkeleton)_ORBIT_skel_small_Bonobo_Unknown_ref;
break;
case 'u':
if(strcmp((opname + 1), "nref")) break;
*impl = (gpointer)servant->vepv->Bonobo_Unknown_epv->unref;
{ORBit_IInterface *volatile _t_=&Bonobo_Unknown__iinterface;*m_data = (gpointer)&_t_->methods._buffer [1];}
return (ORBitSmallSkeleton)_ORBIT_skel_small_Bonobo_Unknown_unref;
break;
default: break; 
}
return NULL;
}

void POA_Evolution_Mail_MessageStream__init(PortableServer_Servant servant,
CORBA_Environment *env)
{
  static PortableServer_ClassInfo class_info = {NULL, (ORBit_small_impl_finder)&get_skel_small_Evolution_Mail_MessageStream, "IDL:Evolution/Mail/MessageStream:1.0", &Evolution_Mail_MessageStream__classid, NULL, &Evolution_Mail_MessageStream__iinterface};
  PortableServer_ServantBase__init (       ((PortableServer_ServantBase *)servant), env);
  POA_Bonobo_Unknown__init(servant, env);
   ORBit_skel_class_register (&class_info,
   (PortableServer_ServantBase *)servant, POA_Evolution_Mail_MessageStream__fini,
   ORBIT_VEPV_OFFSET (POA_Evolution_Mail_MessageStream__vepv, Evolution_Mail_MessageStream_epv),
(CORBA_unsigned_long) Bonobo_Unknown__classid,
ORBIT_VEPV_OFFSET (POA_Evolution_Mail_MessageStream__vepv, Bonobo_Unknown_epv),
   (CORBA_unsigned_long) 0);}

void POA_Evolution_Mail_MessageStream__fini(PortableServer_Servant servant,
CORBA_Environment *env)
{
  POA_Bonobo_Unknown__fini(servant, env);
  PortableServer_ServantBase__fini(servant, env);
}

static ORBitSmallSkeleton get_skel_small_Evolution_Mail_MessageIterator(POA_Evolution_Mail_MessageIterator *servant,
const char *opname,gpointer *m_data, gpointer *impl)
{
switch(opname[0]) {
case 'd':
if(strcmp((opname + 1), "ispose")) break;
*impl = (gpointer)servant->vepv->Evolution_Mail_MessageIterator_epv->dispose;
{ORBit_IInterface *volatile _t_=&Evolution_Mail_MessageIterator__iinterface;*m_data = (gpointer)&_t_->methods._buffer [1];}
return (ORBitSmallSkeleton)_ORBIT_skel_small_Evolution_Mail_MessageIterator_dispose;
break;
case 'n':
if(strcmp((opname + 1), "ext")) break;
*impl = (gpointer)servant->vepv->Evolution_Mail_MessageIterator_epv->next;
{ORBit_IInterface *volatile _t_=&Evolution_Mail_MessageIterator__iinterface;*m_data = (gpointer)&_t_->methods._buffer [0];}
return (ORBitSmallSkeleton)_ORBIT_skel_small_Evolution_Mail_MessageIterator_next;
break;
case 'q':
if(strcmp((opname + 1), "ueryInterface")) break;
*impl = (gpointer)servant->vepv->Bonobo_Unknown_epv->queryInterface;
{ORBit_IInterface *volatile _t_=&Bonobo_Unknown__iinterface;*m_data = (gpointer)&_t_->methods._buffer [2];}
return (ORBitSmallSkeleton)_ORBIT_skel_small_Bonobo_Unknown_queryInterface;
break;
case 'r':
if(strcmp((opname + 1), "ef")) break;
*impl = (gpointer)servant->vepv->Bonobo_Unknown_epv->ref;
{ORBit_IInterface *volatile _t_=&Bonobo_Unknown__iinterface;*m_data = (gpointer)&_t_->methods._buffer [0];}
return (ORBitSmallSkeleton)_ORBIT_skel_small_Bonobo_Unknown_ref;
break;
case 'u':
if(strcmp((opname + 1), "nref")) break;
*impl = (gpointer)servant->vepv->Bonobo_Unknown_epv->unref;
{ORBit_IInterface *volatile _t_=&Bonobo_Unknown__iinterface;*m_data = (gpointer)&_t_->methods._buffer [1];}
return (ORBitSmallSkeleton)_ORBIT_skel_small_Bonobo_Unknown_unref;
break;
default: break; 
}
return NULL;
}

void POA_Evolution_Mail_MessageIterator__init(PortableServer_Servant servant,
CORBA_Environment *env)
{
  static PortableServer_ClassInfo class_info = {NULL, (ORBit_small_impl_finder)&get_skel_small_Evolution_Mail_MessageIterator, "IDL:Evolution/Mail/MessageIterator:1.0", &Evolution_Mail_MessageIterator__classid, NULL, &Evolution_Mail_MessageIterator__iinterface};
  PortableServer_ServantBase__init (       ((PortableServer_ServantBase *)servant), env);
  POA_Bonobo_Unknown__init(servant, env);
   ORBit_skel_class_register (&class_info,
   (PortableServer_ServantBase *)servant, POA_Evolution_Mail_MessageIterator__fini,
   ORBIT_VEPV_OFFSET (POA_Evolution_Mail_MessageIterator__vepv, Evolution_Mail_MessageIterator_epv),
(CORBA_unsigned_long) Bonobo_Unknown__classid,
ORBIT_VEPV_OFFSET (POA_Evolution_Mail_MessageIterator__vepv, Bonobo_Unknown_epv),
   (CORBA_unsigned_long) 0);}

void POA_Evolution_Mail_MessageIterator__fini(PortableServer_Servant servant,
CORBA_Environment *env)
{
  POA_Bonobo_Unknown__fini(servant, env);
  PortableServer_ServantBase__fini(servant, env);
}

static ORBitSmallSkeleton get_skel_small_Evolution_Mail_SessionListener(POA_Evolution_Mail_SessionListener *servant,
const char *opname,gpointer *m_data, gpointer *impl)
{
switch(opname[0]) {
case 'c':
if(strcmp((opname + 1), "hanged")) break;
*impl = (gpointer)servant->vepv->Evolution_Mail_SessionListener_epv->changed;
{ORBit_IInterface *volatile _t_=&Evolution_Mail_SessionListener__iinterface;*m_data = (gpointer)&_t_->methods._buffer [0];}
return (ORBitSmallSkeleton)_ORBIT_skel_small_Evolution_Mail_SessionListener_changed;
break;
case 'q':
if(strcmp((opname + 1), "ueryInterface")) break;
*impl = (gpointer)servant->vepv->Bonobo_Unknown_epv->queryInterface;
{ORBit_IInterface *volatile _t_=&Bonobo_Unknown__iinterface;*m_data = (gpointer)&_t_->methods._buffer [2];}
return (ORBitSmallSkeleton)_ORBIT_skel_small_Bonobo_Unknown_queryInterface;
break;
case 'r':
if(strcmp((opname + 1), "ef")) break;
*impl = (gpointer)servant->vepv->Bonobo_Unknown_epv->ref;
{ORBit_IInterface *volatile _t_=&Bonobo_Unknown__iinterface;*m_data = (gpointer)&_t_->methods._buffer [0];}
return (ORBitSmallSkeleton)_ORBIT_skel_small_Bonobo_Unknown_ref;
break;
case 's':
if(strcmp((opname + 1), "hutdown")) break;
*impl = (gpointer)servant->vepv->Evolution_Mail_SessionListener_epv->shutdown;
{ORBit_IInterface *volatile _t_=&Evolution_Mail_SessionListener__iinterface;*m_data = (gpointer)&_t_->methods._buffer [1];}
return (ORBitSmallSkeleton)_ORBIT_skel_small_Evolution_Mail_SessionListener_shutdown;
break;
case 'u':
if(strcmp((opname + 1), "nref")) break;
*impl = (gpointer)servant->vepv->Bonobo_Unknown_epv->unref;
{ORBit_IInterface *volatile _t_=&Bonobo_Unknown__iinterface;*m_data = (gpointer)&_t_->methods._buffer [1];}
return (ORBitSmallSkeleton)_ORBIT_skel_small_Bonobo_Unknown_unref;
break;
default: break; 
}
return NULL;
}

void POA_Evolution_Mail_SessionListener__init(PortableServer_Servant servant,
CORBA_Environment *env)
{
  static PortableServer_ClassInfo class_info = {NULL, (ORBit_small_impl_finder)&get_skel_small_Evolution_Mail_SessionListener, "IDL:Evolution/Mail/SessionListener:1.0", &Evolution_Mail_SessionListener__classid, NULL, &Evolution_Mail_SessionListener__iinterface};
  PortableServer_ServantBase__init (       ((PortableServer_ServantBase *)servant), env);
  POA_Bonobo_Unknown__init(servant, env);
   ORBit_skel_class_register (&class_info,
   (PortableServer_ServantBase *)servant, POA_Evolution_Mail_SessionListener__fini,
   ORBIT_VEPV_OFFSET (POA_Evolution_Mail_SessionListener__vepv, Evolution_Mail_SessionListener_epv),
(CORBA_unsigned_long) Bonobo_Unknown__classid,
ORBIT_VEPV_OFFSET (POA_Evolution_Mail_SessionListener__vepv, Bonobo_Unknown_epv),
   (CORBA_unsigned_long) 0);}

void POA_Evolution_Mail_SessionListener__fini(PortableServer_Servant servant,
CORBA_Environment *env)
{
  POA_Bonobo_Unknown__fini(servant, env);
  PortableServer_ServantBase__fini(servant, env);
}

static ORBitSmallSkeleton get_skel_small_Evolution_Mail_StoreListener(POA_Evolution_Mail_StoreListener *servant,
const char *opname,gpointer *m_data, gpointer *impl)
{
switch(opname[0]) {
case 'c':
if(strcmp((opname + 1), "hanged")) break;
*impl = (gpointer)servant->vepv->Evolution_Mail_StoreListener_epv->changed;
{ORBit_IInterface *volatile _t_=&Evolution_Mail_StoreListener__iinterface;*m_data = (gpointer)&_t_->methods._buffer [0];}
return (ORBitSmallSkeleton)_ORBIT_skel_small_Evolution_Mail_StoreListener_changed;
break;
case 'q':
if(strcmp((opname + 1), "ueryInterface")) break;
*impl = (gpointer)servant->vepv->Bonobo_Unknown_epv->queryInterface;
{ORBit_IInterface *volatile _t_=&Bonobo_Unknown__iinterface;*m_data = (gpointer)&_t_->methods._buffer [2];}
return (ORBitSmallSkeleton)_ORBIT_skel_small_Bonobo_Unknown_queryInterface;
break;
case 'r':
if(strcmp((opname + 1), "ef")) break;
*impl = (gpointer)servant->vepv->Bonobo_Unknown_epv->ref;
{ORBit_IInterface *volatile _t_=&Bonobo_Unknown__iinterface;*m_data = (gpointer)&_t_->methods._buffer [0];}
return (ORBitSmallSkeleton)_ORBIT_skel_small_Bonobo_Unknown_ref;
break;
case 'u':
if(strcmp((opname + 1), "nref")) break;
*impl = (gpointer)servant->vepv->Bonobo_Unknown_epv->unref;
{ORBit_IInterface *volatile _t_=&Bonobo_Unknown__iinterface;*m_data = (gpointer)&_t_->methods._buffer [1];}
return (ORBitSmallSkeleton)_ORBIT_skel_small_Bonobo_Unknown_unref;
break;
default: break; 
}
return NULL;
}

void POA_Evolution_Mail_StoreListener__init(PortableServer_Servant servant,
CORBA_Environment *env)
{
  static PortableServer_ClassInfo class_info = {NULL, (ORBit_small_impl_finder)&get_skel_small_Evolution_Mail_StoreListener, "IDL:Evolution/Mail/StoreListener:1.0", &Evolution_Mail_StoreListener__classid, NULL, &Evolution_Mail_StoreListener__iinterface};
  PortableServer_ServantBase__init (       ((PortableServer_ServantBase *)servant), env);
  POA_Bonobo_Unknown__init(servant, env);
   ORBit_skel_class_register (&class_info,
   (PortableServer_ServantBase *)servant, POA_Evolution_Mail_StoreListener__fini,
   ORBIT_VEPV_OFFSET (POA_Evolution_Mail_StoreListener__vepv, Evolution_Mail_StoreListener_epv),
(CORBA_unsigned_long) Bonobo_Unknown__classid,
ORBIT_VEPV_OFFSET (POA_Evolution_Mail_StoreListener__vepv, Bonobo_Unknown_epv),
   (CORBA_unsigned_long) 0);}

void POA_Evolution_Mail_StoreListener__fini(PortableServer_Servant servant,
CORBA_Environment *env)
{
  POA_Bonobo_Unknown__fini(servant, env);
  PortableServer_ServantBase__fini(servant, env);
}

static ORBitSmallSkeleton get_skel_small_Evolution_Mail_FolderListener(POA_Evolution_Mail_FolderListener *servant,
const char *opname,gpointer *m_data, gpointer *impl)
{
switch(opname[0]) {
case 'c':
if(strcmp((opname + 1), "hanged")) break;
*impl = (gpointer)servant->vepv->Evolution_Mail_FolderListener_epv->changed;
{ORBit_IInterface *volatile _t_=&Evolution_Mail_FolderListener__iinterface;*m_data = (gpointer)&_t_->methods._buffer [0];}
return (ORBitSmallSkeleton)_ORBIT_skel_small_Evolution_Mail_FolderListener_changed;
break;
case 'q':
if(strcmp((opname + 1), "ueryInterface")) break;
*impl = (gpointer)servant->vepv->Bonobo_Unknown_epv->queryInterface;
{ORBit_IInterface *volatile _t_=&Bonobo_Unknown__iinterface;*m_data = (gpointer)&_t_->methods._buffer [2];}
return (ORBitSmallSkeleton)_ORBIT_skel_small_Bonobo_Unknown_queryInterface;
break;
case 'r':
if(strcmp((opname + 1), "ef")) break;
*impl = (gpointer)servant->vepv->Bonobo_Unknown_epv->ref;
{ORBit_IInterface *volatile _t_=&Bonobo_Unknown__iinterface;*m_data = (gpointer)&_t_->methods._buffer [0];}
return (ORBitSmallSkeleton)_ORBIT_skel_small_Bonobo_Unknown_ref;
break;
case 'u':
if(strcmp((opname + 1), "nref")) break;
*impl = (gpointer)servant->vepv->Bonobo_Unknown_epv->unref;
{ORBit_IInterface *volatile _t_=&Bonobo_Unknown__iinterface;*m_data = (gpointer)&_t_->methods._buffer [1];}
return (ORBitSmallSkeleton)_ORBIT_skel_small_Bonobo_Unknown_unref;
break;
default: break; 
}
return NULL;
}

void POA_Evolution_Mail_FolderListener__init(PortableServer_Servant servant,
CORBA_Environment *env)
{
  static PortableServer_ClassInfo class_info = {NULL, (ORBit_small_impl_finder)&get_skel_small_Evolution_Mail_FolderListener, "IDL:Evolution/Mail/FolderListener:1.0", &Evolution_Mail_FolderListener__classid, NULL, &Evolution_Mail_FolderListener__iinterface};
  PortableServer_ServantBase__init (       ((PortableServer_ServantBase *)servant), env);
  POA_Bonobo_Unknown__init(servant, env);
   ORBit_skel_class_register (&class_info,
   (PortableServer_ServantBase *)servant, POA_Evolution_Mail_FolderListener__fini,
   ORBIT_VEPV_OFFSET (POA_Evolution_Mail_FolderListener__vepv, Evolution_Mail_FolderListener_epv),
(CORBA_unsigned_long) Bonobo_Unknown__classid,
ORBIT_VEPV_OFFSET (POA_Evolution_Mail_FolderListener__vepv, Bonobo_Unknown_epv),
   (CORBA_unsigned_long) 0);}

void POA_Evolution_Mail_FolderListener__fini(PortableServer_Servant servant,
CORBA_Environment *env)
{
  POA_Bonobo_Unknown__fini(servant, env);
  PortableServer_ServantBase__fini(servant, env);
}

static ORBitSmallSkeleton get_skel_small_Evolution_Mail_Session(POA_Evolution_Mail_Session *servant,
const char *opname,gpointer *m_data, gpointer *impl)
{
switch(opname[0]) {
case 'a':
if(strcmp((opname + 1), "ddListener")) break;
*impl = (gpointer)servant->vepv->Evolution_Mail_Session_epv->addListener;
{ORBit_IInterface *volatile _t_=&Evolution_Mail_Session__iinterface;*m_data = (gpointer)&_t_->methods._buffer [2];}
return (ORBitSmallSkeleton)_ORBIT_skel_small_Evolution_Mail_Session_addListener;
break;
case 'g':
switch(opname[1]) {
case 'e':
switch(opname[2]) {
case 't':
switch(opname[3]) {
case 'P':
if(strcmp((opname + 4), "roperties")) break;
*impl = (gpointer)servant->vepv->Evolution_Mail_Session_epv->getProperties;
{ORBit_IInterface *volatile _t_=&Evolution_Mail_Session__iinterface;*m_data = (gpointer)&_t_->methods._buffer [0];}
return (ORBitSmallSkeleton)_ORBIT_skel_small_Evolution_Mail_Session_getProperties;
break;
case 'S':
if(strcmp((opname + 4), "tores")) break;
*impl = (gpointer)servant->vepv->Evolution_Mail_Session_epv->getStores;
{ORBit_IInterface *volatile _t_=&Evolution_Mail_Session__iinterface;*m_data = (gpointer)&_t_->methods._buffer [1];}
return (ORBitSmallSkeleton)_ORBIT_skel_small_Evolution_Mail_Session_getStores;
break;
default: break; 
}
break;
default: break; 
}
break;
default: break; 
}
break;
case 'q':
if(strcmp((opname + 1), "ueryInterface")) break;
*impl = (gpointer)servant->vepv->Bonobo_Unknown_epv->queryInterface;
{ORBit_IInterface *volatile _t_=&Bonobo_Unknown__iinterface;*m_data = (gpointer)&_t_->methods._buffer [2];}
return (ORBitSmallSkeleton)_ORBIT_skel_small_Bonobo_Unknown_queryInterface;
break;
case 'r':
switch(opname[1]) {
case 'e':
switch(opname[2]) {
case 'f':
if(strcmp((opname + 3), "")) break;
*impl = (gpointer)servant->vepv->Bonobo_Unknown_epv->ref;
{ORBit_IInterface *volatile _t_=&Bonobo_Unknown__iinterface;*m_data = (gpointer)&_t_->methods._buffer [0];}
return (ORBitSmallSkeleton)_ORBIT_skel_small_Bonobo_Unknown_ref;
break;
case 'm':
if(strcmp((opname + 3), "oveListener")) break;
*impl = (gpointer)servant->vepv->Evolution_Mail_Session_epv->removeListener;
{ORBit_IInterface *volatile _t_=&Evolution_Mail_Session__iinterface;*m_data = (gpointer)&_t_->methods._buffer [3];}
return (ORBitSmallSkeleton)_ORBIT_skel_small_Evolution_Mail_Session_removeListener;
break;
default: break; 
}
break;
default: break; 
}
break;
case 'u':
if(strcmp((opname + 1), "nref")) break;
*impl = (gpointer)servant->vepv->Bonobo_Unknown_epv->unref;
{ORBit_IInterface *volatile _t_=&Bonobo_Unknown__iinterface;*m_data = (gpointer)&_t_->methods._buffer [1];}
return (ORBitSmallSkeleton)_ORBIT_skel_small_Bonobo_Unknown_unref;
break;
default: break; 
}
return NULL;
}

void POA_Evolution_Mail_Session__init(PortableServer_Servant servant,
CORBA_Environment *env)
{
  static PortableServer_ClassInfo class_info = {NULL, (ORBit_small_impl_finder)&get_skel_small_Evolution_Mail_Session, "IDL:Evolution/Mail/Session:1.0", &Evolution_Mail_Session__classid, NULL, &Evolution_Mail_Session__iinterface};
  PortableServer_ServantBase__init (       ((PortableServer_ServantBase *)servant), env);
  POA_Bonobo_Unknown__init(servant, env);
   ORBit_skel_class_register (&class_info,
   (PortableServer_ServantBase *)servant, POA_Evolution_Mail_Session__fini,
   ORBIT_VEPV_OFFSET (POA_Evolution_Mail_Session__vepv, Evolution_Mail_Session_epv),
(CORBA_unsigned_long) Bonobo_Unknown__classid,
ORBIT_VEPV_OFFSET (POA_Evolution_Mail_Session__vepv, Bonobo_Unknown_epv),
   (CORBA_unsigned_long) 0);}

void POA_Evolution_Mail_Session__fini(PortableServer_Servant servant,
CORBA_Environment *env)
{
  POA_Bonobo_Unknown__fini(servant, env);
  PortableServer_ServantBase__fini(servant, env);
}

static ORBitSmallSkeleton get_skel_small_Evolution_Mail_Store(POA_Evolution_Mail_Store *servant,
const char *opname,gpointer *m_data, gpointer *impl)
{
switch(opname[0]) {
case 'g':
switch(opname[1]) {
case 'e':
switch(opname[2]) {
case 't':
switch(opname[3]) {
case 'F':
if(strcmp((opname + 4), "olders")) break;
*impl = (gpointer)servant->vepv->Evolution_Mail_Store_epv->getFolders;
{ORBit_IInterface *volatile _t_=&Evolution_Mail_Store__iinterface;*m_data = (gpointer)&_t_->methods._buffer [1];}
return (ORBitSmallSkeleton)_ORBIT_skel_small_Evolution_Mail_Store_getFolders;
break;
case 'P':
if(strcmp((opname + 4), "roperties")) break;
*impl = (gpointer)servant->vepv->Evolution_Mail_Store_epv->getProperties;
{ORBit_IInterface *volatile _t_=&Evolution_Mail_Store__iinterface;*m_data = (gpointer)&_t_->methods._buffer [0];}
return (ORBitSmallSkeleton)_ORBIT_skel_small_Evolution_Mail_Store_getProperties;
break;
default: break; 
}
break;
default: break; 
}
break;
default: break; 
}
break;
case 'q':
if(strcmp((opname + 1), "ueryInterface")) break;
*impl = (gpointer)servant->vepv->Bonobo_Unknown_epv->queryInterface;
{ORBit_IInterface *volatile _t_=&Bonobo_Unknown__iinterface;*m_data = (gpointer)&_t_->methods._buffer [2];}
return (ORBitSmallSkeleton)_ORBIT_skel_small_Bonobo_Unknown_queryInterface;
break;
case 'r':
if(strcmp((opname + 1), "ef")) break;
*impl = (gpointer)servant->vepv->Bonobo_Unknown_epv->ref;
{ORBit_IInterface *volatile _t_=&Bonobo_Unknown__iinterface;*m_data = (gpointer)&_t_->methods._buffer [0];}
return (ORBitSmallSkeleton)_ORBIT_skel_small_Bonobo_Unknown_ref;
break;
case 's':
if(strcmp((opname + 1), "endMessage")) break;
*impl = (gpointer)servant->vepv->Evolution_Mail_Store_epv->sendMessage;
{ORBit_IInterface *volatile _t_=&Evolution_Mail_Store__iinterface;*m_data = (gpointer)&_t_->methods._buffer [2];}
return (ORBitSmallSkeleton)_ORBIT_skel_small_Evolution_Mail_Store_sendMessage;
break;
case 'u':
if(strcmp((opname + 1), "nref")) break;
*impl = (gpointer)servant->vepv->Bonobo_Unknown_epv->unref;
{ORBit_IInterface *volatile _t_=&Bonobo_Unknown__iinterface;*m_data = (gpointer)&_t_->methods._buffer [1];}
return (ORBitSmallSkeleton)_ORBIT_skel_small_Bonobo_Unknown_unref;
break;
default: break; 
}
return NULL;
}

void POA_Evolution_Mail_Store__init(PortableServer_Servant servant,
CORBA_Environment *env)
{
  static PortableServer_ClassInfo class_info = {NULL, (ORBit_small_impl_finder)&get_skel_small_Evolution_Mail_Store, "IDL:Evolution/Mail/Store:1.0", &Evolution_Mail_Store__classid, NULL, &Evolution_Mail_Store__iinterface};
  PortableServer_ServantBase__init (       ((PortableServer_ServantBase *)servant), env);
  POA_Bonobo_Unknown__init(servant, env);
   ORBit_skel_class_register (&class_info,
   (PortableServer_ServantBase *)servant, POA_Evolution_Mail_Store__fini,
   ORBIT_VEPV_OFFSET (POA_Evolution_Mail_Store__vepv, Evolution_Mail_Store_epv),
(CORBA_unsigned_long) Bonobo_Unknown__classid,
ORBIT_VEPV_OFFSET (POA_Evolution_Mail_Store__vepv, Bonobo_Unknown_epv),
   (CORBA_unsigned_long) 0);}

void POA_Evolution_Mail_Store__fini(PortableServer_Servant servant,
CORBA_Environment *env)
{
  POA_Bonobo_Unknown__fini(servant, env);
  PortableServer_ServantBase__fini(servant, env);
}

static ORBitSmallSkeleton get_skel_small_Evolution_Mail_Folder(POA_Evolution_Mail_Folder *servant,
const char *opname,gpointer *m_data, gpointer *impl)
{
switch(opname[0]) {
case 'a':
if(strcmp((opname + 1), "ppendMessage")) break;
*impl = (gpointer)servant->vepv->Evolution_Mail_Folder_epv->appendMessage;
{ORBit_IInterface *volatile _t_=&Evolution_Mail_Folder__iinterface;*m_data = (gpointer)&_t_->methods._buffer [4];}
return (ORBitSmallSkeleton)_ORBIT_skel_small_Evolution_Mail_Folder_appendMessage;
break;
case 'c':
if(strcmp((opname + 1), "hangeMessages")) break;
*impl = (gpointer)servant->vepv->Evolution_Mail_Folder_epv->changeMessages;
{ORBit_IInterface *volatile _t_=&Evolution_Mail_Folder__iinterface;*m_data = (gpointer)&_t_->methods._buffer [2];}
return (ORBitSmallSkeleton)_ORBIT_skel_small_Evolution_Mail_Folder_changeMessages;
break;
case 'g':
switch(opname[1]) {
case 'e':
switch(opname[2]) {
case 't':
switch(opname[3]) {
case 'M':
switch(opname[4]) {
case 'e':
switch(opname[5]) {
case 's':
switch(opname[6]) {
case 's':
switch(opname[7]) {
case 'a':
switch(opname[8]) {
case 'g':
switch(opname[9]) {
case 'e':
switch(opname[10]) {
case '\0':
*impl = (gpointer)servant->vepv->Evolution_Mail_Folder_epv->getMessage;
{ORBit_IInterface *volatile _t_=&Evolution_Mail_Folder__iinterface;*m_data = (gpointer)&_t_->methods._buffer [3];}
return (ORBitSmallSkeleton)_ORBIT_skel_small_Evolution_Mail_Folder_getMessage;
break;
case 's':
if(strcmp((opname + 11), "")) break;
*impl = (gpointer)servant->vepv->Evolution_Mail_Folder_epv->getMessages;
{ORBit_IInterface *volatile _t_=&Evolution_Mail_Folder__iinterface;*m_data = (gpointer)&_t_->methods._buffer [1];}
return (ORBitSmallSkeleton)_ORBIT_skel_small_Evolution_Mail_Folder_getMessages;
break;
default: break; 
}
break;
default: break; 
}
break;
default: break; 
}
break;
default: break; 
}
break;
default: break; 
}
break;
default: break; 
}
break;
default: break; 
}
break;
case 'P':
if(strcmp((opname + 4), "roperties")) break;
*impl = (gpointer)servant->vepv->Evolution_Mail_Folder_epv->getProperties;
{ORBit_IInterface *volatile _t_=&Evolution_Mail_Folder__iinterface;*m_data = (gpointer)&_t_->methods._buffer [0];}
return (ORBitSmallSkeleton)_ORBIT_skel_small_Evolution_Mail_Folder_getProperties;
break;
default: break; 
}
break;
default: break; 
}
break;
default: break; 
}
break;
case 'q':
if(strcmp((opname + 1), "ueryInterface")) break;
*impl = (gpointer)servant->vepv->Bonobo_Unknown_epv->queryInterface;
{ORBit_IInterface *volatile _t_=&Bonobo_Unknown__iinterface;*m_data = (gpointer)&_t_->methods._buffer [2];}
return (ORBitSmallSkeleton)_ORBIT_skel_small_Bonobo_Unknown_queryInterface;
break;
case 'r':
if(strcmp((opname + 1), "ef")) break;
*impl = (gpointer)servant->vepv->Bonobo_Unknown_epv->ref;
{ORBit_IInterface *volatile _t_=&Bonobo_Unknown__iinterface;*m_data = (gpointer)&_t_->methods._buffer [0];}
return (ORBitSmallSkeleton)_ORBIT_skel_small_Bonobo_Unknown_ref;
break;
case 'u':
if(strcmp((opname + 1), "nref")) break;
*impl = (gpointer)servant->vepv->Bonobo_Unknown_epv->unref;
{ORBit_IInterface *volatile _t_=&Bonobo_Unknown__iinterface;*m_data = (gpointer)&_t_->methods._buffer [1];}
return (ORBitSmallSkeleton)_ORBIT_skel_small_Bonobo_Unknown_unref;
break;
default: break; 
}
return NULL;
}

void POA_Evolution_Mail_Folder__init(PortableServer_Servant servant,
CORBA_Environment *env)
{
  static PortableServer_ClassInfo class_info = {NULL, (ORBit_small_impl_finder)&get_skel_small_Evolution_Mail_Folder, "IDL:Evolution/Mail/Folder:1.0", &Evolution_Mail_Folder__classid, NULL, &Evolution_Mail_Folder__iinterface};
  PortableServer_ServantBase__init (       ((PortableServer_ServantBase *)servant), env);
  POA_Bonobo_Unknown__init(servant, env);
   ORBit_skel_class_register (&class_info,
   (PortableServer_ServantBase *)servant, POA_Evolution_Mail_Folder__fini,
   ORBIT_VEPV_OFFSET (POA_Evolution_Mail_Folder__vepv, Evolution_Mail_Folder_epv),
(CORBA_unsigned_long) Bonobo_Unknown__classid,
ORBIT_VEPV_OFFSET (POA_Evolution_Mail_Folder__vepv, Bonobo_Unknown_epv),
   (CORBA_unsigned_long) 0);}

void POA_Evolution_Mail_Folder__fini(PortableServer_Servant servant,
CORBA_Environment *env)
{
  POA_Bonobo_Unknown__fini(servant, env);
  PortableServer_ServantBase__fini(servant, env);
}

